/************************************************************************/
/* Routines pour communiquer avec le port serie 2                       */
/* PLLx16    Format :38400bds 8bits pas de parit                       */
/* Rception et stockage des caractres dans le BUFFER par interruption */
/* Les autres routines utilisent le BUFFER          J. Senlis  2005     */
/************************************************************************/
#include "p30f6014.h"       /*definitions des registres etc...*/
extern float fcy;
/**********Variables globales********************************************/
char BUFFER[255];			/* pour la rception srie*/
int  PT1BUF=0 ;				/* pointeur  criture     */
int  PT2BUF=0;				/*   "        lecture     */  
void init_uart(void);
char lit_char(void);
char get_char(void);
long int lit_int(void);
long int lit_dec(void);
long double lit_double(void);
void envoi_long_dec(long int a);
void envoi_int(unsigned int a);
void envoi_dec(int aa);
void envoi_char(char a);
void envoi_chaine(char* ch);
void envoi_menu(void);
void crlf(void);
void ok(void);

/**********Ecriture des fonctions****************************************/
void init_uart(void)
	{asm("disi #32");
	asm("bset INTCON1,#15");
	U2MODE = 0x0000;U2STA=0;
	U2MODE = 0x8020  ;
	//U2BRG=(((4*7372800/(38400)) / 16) - 1);//(pllx16)   
	//U2BRG=(((4*8000000/(38400)) / 16) - 1);//(pllx16)
	U2BRG=(((fcy/(38400)) / 16) - 1);//(pllx16)
	U2STA =  0x8400 ;
	asm("bset IPC6,#0");
	asm("bset IPC6,#1");
	asm("bset IPC6,#2");
	asm("bclr IFS1,#8") ;
	asm("bset IEC1,#8") ;
	}

void envoi_char(char a)
	{while ((U2STA & 0x0100)==0){};
    U2TXREG= a;
	} 

char lit_char(void)                   
	{/*Attend un caractre  */
	char a;
	while (PT1BUF==PT2BUF){}
	a= BUFFER[PT2BUF];
	PT2BUF++;
	if (PT2BUF==255) PT2BUF=0;
	return (a);
	}

char get_char(void)                   
	{/*n'attend pas , retourne 0 si le BUFFER est vide */
	char a;
	a=0;
	if(PT1BUF==PT2BUF) a=0;
	else 
		{a= BUFFER[PT2BUF];
		PT2BUF++;
		if (PT2BUF==255) PT2BUF=0;
		}
	return (a);
}

void lit_chaine(char *ch)
	{int i=0;char a;
    while((a=lit_char())!=0x0d)
	    {if(a==8)
			{if (i>0){i--;envoi_char(' ');envoi_char(8);}}
		else{ch[i]=a;i++;}
		}
   ch[i]=0;
   }
void ok(void)
{
 envoi_char('>');
 envoi_char('>'); 
}	

void crlf(void)
	{
	envoi_char(0x0d);
	envoi_char(0x0a);
	}	

void envoi_chaine(char* ch)
	{int i=0;char a;
	while((a=ch[i])!='\0') 
		{envoi_char(a); 
		i++;            		
		} 
    }    

void envoi_int(unsigned int a)
	{int i;
	char c[16]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'} ;
	i=a >> 12;a =a%4096;
	envoi_char(c[i]);
	i=a >> 8;a =a%256;
	envoi_char(c[i]);
	i=a >> 4;a =a%16;
	envoi_char(c[i]);
	envoi_char(c[a]);
	}

void envoi_dec(int aa)
	{int d,i,j,k,a,l;
	char c[10]={'0','1','2','3','4','5','6','7','8','9'} ;
	a=aa;
	if (aa<0) {envoi_char('-');a=abs(aa);}
	j=0; 
	for (k=4;k>=0;k--)
		{d=1;
		for (l=0;l<k;l++) 
			{d=d*10;}
		i=a/d;a=a%d;
		 if (j!=0 || i!=0||k==0) {j=1;envoi_char(c[i]);}
		 }
	}
			
void envoi_long_dec(long int aa)
	{long int d,a;
	int k,l,i,j;
	char c[10]={'0','1','2','3','4','5','6','7','8','9'} ;
	a=aa;
	if (aa<0) {envoi_char('-');a=abs(aa);}
	else envoi_char('+');
	j=0; 
	for (k=8;k>=0;k--)
		{d=1;
		for (l=0;l<k;l++) 
			{d=d*10;}
		i=a/d;a=a%d;
		 if (j!=0 || i!=0||k==0) {j=1;envoi_char(c[i]);}
		 }
	}
			
unsigned int chiffre(char a)
	/* ;Conversion 0..F cod ASCII en 0..F cod binaire */
	{int x;
	x=toupper(a);   
	if(x<0x30) x=0;
    else 
    	{x=x-0x30;
    	if (x>9)
    		{if(x<17) x=0; 
    		 else 
    	 		{x=x-7;
    	 	     if (x>16) x=0; 
    	 	    }
    	 	}
    	 }  
    return (x);	 
	}    

long int lit_int(void)
	{long int x=0,y=0;
	char a;
	int i=0;
	for (i=0;i<4;i++)
		{a=lit_char();
		y=chiffre(a);
		x=(x*16)+y;
        }
    return(x); 
    }                    

long int lit_dec(void)
	{long int x=0,y=0,signe=1;
	char a;
	while((a=lit_char())!=0x0d) 
		{if (a=='-') signe=-1;
		 if (a=='+') signe=1;
		 if (a==8) x=0; 
		 if ((a>=0x30)&&(a<=0x39))
			{y=chiffre(a);
			x=(x*10)+y;
			}
        }   
    x=signe*x;    
    return(x); 
     }                    

long double lit_double(void)
	{unsigned int x=0,y=0;
	char a;
	int i=0;
	for (i=0;i<5;i++)
		{a=lit_char();
		y=chiffre(a);
		x=(x*10)+y;
        }
    return((long double)x); 
    }                    

/***********Routine de traitement de l'interruption caractre reu sur le port srie 2******/

void __attribute__((__interrupt__)) _U2RXInterrupt(void)
	{/*Les caractres reus sont placs dans un buffer modulo 255 */
    int a;
	a= (U2RXREG & 0x00FF);
 	BUFFER[PT1BUF]=a;
    PT1BUF++;
    if (PT1BUF==255) PT1BUF=0; 
	envoi_char(a);
	asm("bclr IFS1,#8") ;
	}
